package javax.swing;

import java.awt.EventQueue;
import java.awt.Composite;
import java.awt.Color;
import java.awt.AlphaComposite;
import java.awt.Graphics2D;
import sun.java2d.SunGraphicsEnvironment;
import sun.security.action.GetPropertyAction;
import sun.swing.SwingAccessor;
import sun.misc.SharedSecrets;
import com.sun.java.swing.SwingUtilities3;
import java.awt.event.InvocationEvent;
import java.awt.GraphicsDevice;
import java.awt.HeadlessException;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Graphics;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.HashSet;
import java.awt.Toolkit;
import sun.awt.AWTAccessor;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.LinkedList;
import java.util.Iterator;
import java.awt.Frame;
import java.applet.Applet;
import java.awt.Window;
import sun.awt.SunToolkit;
import java.util.IdentityHashMap;
import java.util.ArrayList;
import java.util.HashMap;
import sun.awt.AppContext;
import sun.swing.SwingUtilities2;
import sun.awt.DisplayChangedListener;
import sun.misc.JavaSecurityAccess;
import java.awt.Dimension;
import java.util.List;
import java.awt.Component;
import java.awt.Rectangle;
import java.awt.Container;
import java.awt.image.VolatileImage;
import java.awt.GraphicsConfiguration;
import java.util.Map;

public class RepaintManager
{
    static final boolean HANDLE_TOP_LEVEL_PAINT;
    private static final short BUFFER_STRATEGY_NOT_SPECIFIED = 0;
    private static final short BUFFER_STRATEGY_SPECIFIED_ON = 1;
    private static final short BUFFER_STRATEGY_SPECIFIED_OFF = 2;
    private static final short BUFFER_STRATEGY_TYPE;
    private Map<GraphicsConfiguration, VolatileImage> volatileMap;
    private Map<Container, Rectangle> hwDirtyComponents;
    private Map<Component, Rectangle> dirtyComponents;
    private Map<Component, Rectangle> tmpDirtyComponents;
    private List<Component> invalidComponents;
    private List<Runnable> runnableList;
    boolean doubleBufferingEnabled;
    private Dimension doubleBufferMaxSize;
    RepaintManager.DoubleBufferInfo standardDoubleBuffer;
    private RepaintManager.PaintManager paintManager;
    private static final Object repaintManagerKey;
    static boolean volatileImageBufferEnabled;
    private static final int volatileBufferType;
    private static boolean nativeDoubleBuffering;
    private static final int VOLATILE_LOOP_MAX = 2;
    private int paintDepth;
    private short bufferStrategyType;
    private boolean painting;
    private JComponent repaintRoot;
    private Thread paintThread;
    private final RepaintManager.ProcessingRunnable processingRunnable;
    private static final JavaSecurityAccess javaSecurityAccess;
    private static final DisplayChangedListener displayChangedHandler;
    Rectangle tmp;
    private List<SwingUtilities2.RepaintListener> repaintListeners;
    
    public static RepaintManager currentManager(Component component) {
        return currentManager(AppContext.getAppContext());
    }
    
    static RepaintManager currentManager(AppContext appContext) {
        RepaintManager value = (RepaintManager)appContext.get(RepaintManager.repaintManagerKey);
        if (value == null) {
            value = new RepaintManager(RepaintManager.BUFFER_STRATEGY_TYPE);
            appContext.put(RepaintManager.repaintManagerKey, value);
            return value;
        }
        return value;
    }
    
    public static RepaintManager currentManager(JComponent component) {
        return currentManager(component);
    }
    
    public static void setCurrentManager(RepaintManager repaintManager) {
        if (repaintManager != null) {
            SwingUtilities.appContextPut(RepaintManager.repaintManagerKey, repaintManager);
        }
        else {
            SwingUtilities.appContextRemove(RepaintManager.repaintManagerKey);
        }
    }
    
    public RepaintManager() {
        this((short)2);
    }
    
    private RepaintManager(short bufferStrategyType) {
        this.volatileMap = new HashMap(1);
        this.doubleBufferingEnabled = true;
        this.paintDepth = 0;
        this.tmp = new Rectangle();
        this.repaintListeners = new ArrayList(1);
        this.doubleBufferingEnabled = !RepaintManager.nativeDoubleBuffering;
        synchronized (this) {
            this.dirtyComponents = new IdentityHashMap();
            this.tmpDirtyComponents = new IdentityHashMap();
            this.bufferStrategyType = bufferStrategyType;
            this.hwDirtyComponents = new IdentityHashMap();
        }
        this.processingRunnable = new RepaintManager.ProcessingRunnable();
    }
    
    private void displayChanged() {
        this.clearImages();
    }
    
    public synchronized void addInvalidComponent(JComponent target) {
        RepaintManager delegate = this.getDelegate(target);
        if (delegate != null) {
            delegate.addInvalidComponent(target);
            return;
        }
        Container validateRoot = SwingUtilities.getValidateRoot(target, true);
        if (validateRoot == null) {
            return;
        }
        if (this.invalidComponents == null) {
            this.invalidComponents = new ArrayList();
        }
        else {
            int size = this.invalidComponents.size();
            int i = 0;
            while (i < size) {
                if (validateRoot == this.invalidComponents.get(i)) {
                    return;
                }
                ++i;
            }
        }
        this.invalidComponents.add(validateRoot);
        this.scheduleProcessingRunnable(SunToolkit.targetToAppContext(target));
        return;
    }
    
    public synchronized void removeInvalidComponent(JComponent component) {
        RepaintManager delegate = this.getDelegate(component);
        if (delegate != null) {
            delegate.removeInvalidComponent(component);
            return;
        }
        if (this.invalidComponents != null) {
            int index = this.invalidComponents.indexOf(component);
            if (index != -1) {
                this.invalidComponents.remove(index);
            }
        }
        return;
    }
    
    private void addDirtyRegion0(Container parent, int x, int y, int width, int height) {
        if (width <= 0 || height <= 0 || parent == null) {
            return;
        }
        if (parent.getWidth() <= 0 || parent.getHeight() <= 0) {
            return;
        }
        if (this.extendDirtyRegion(parent, x, y, width, height)) {
            return;
        }
        Container container = null;
        while (parent != null) {
            if (!parent.isVisible() || parent.getPeer() == null) {
                return;
            }
            if (parent instanceof Window || parent instanceof Applet) {
                if (parent instanceof Frame && (((Frame)parent).getExtendedState() & 0x1) == 0x1) {
                    return;
                }
                container = parent;
                break;
            }
            else {
                parent = parent.getParent();
            }
        }
        if (container == null) {
            return;
        }
        synchronized (this) {
            if (this.extendDirtyRegion(parent, x, y, width, height)) {
                return;
            }
            this.dirtyComponents.put(parent, new Rectangle(x, y, width, height));
        }
        this.scheduleProcessingRunnable(SunToolkit.targetToAppContext(parent));
        return;
    }
    
    public void addDirtyRegion(JComponent component, int n, int n2, int n3, int n4) {
        RepaintManager delegate = this.getDelegate(component);
        if (delegate != null) {
            delegate.addDirtyRegion(component, n, n2, n3, n4);
            return;
        }
        this.addDirtyRegion0(component, n, n2, n3, n4);
        return;
    }
    
    public void addDirtyRegion(Window window, int n, int n2, int n3, int n4) {
        this.addDirtyRegion0(window, n, n2, n3, n4);
    }
    
    public void addDirtyRegion(Applet applet, int n, int n2, int n3, int n4) {
        this.addDirtyRegion0(applet, n, n2, n3, n4);
    }
    
    void scheduleHeavyWeightPaints() {
        Map<Object, Object> hwDirtyComponents;
        synchronized (this) {
            if (this.hwDirtyComponents.size() == 0) {
                return;
            }
            hwDirtyComponents = this.hwDirtyComponents;
            this.hwDirtyComponents = new IdentityHashMap();
        }
        for (Container container : hwDirtyComponents.keySet()) {
            Rectangle rectangle = (Rectangle)hwDirtyComponents.get(container);
            if ((container instanceof Window)) {
                this.addDirtyRegion((Window)container, rectangle.x, rectangle.y, rectangle.width, rectangle.height);
            }
            else if ((container instanceof Applet)) {
                this.addDirtyRegion((Applet)container, rectangle.x, rectangle.y, rectangle.width, rectangle.height);
            }
            else {
                this.addDirtyRegion0(container, rectangle.x, rectangle.y, rectangle.width, rectangle.height);
            }
        }
    }
    
    void nativeAddDirtyRegion(AppContext appContext, Container container, int x, int y, int width, int height) {
        if (width > 0 && height > 0) {
            synchronized (this) {
                Rectangle rectangle = (Rectangle)this.hwDirtyComponents.get(container);
                if (rectangle == null) {
                    this.hwDirtyComponents.put(container, new Rectangle(x, y, width, height));
                }
                else {
                    this.hwDirtyComponents.put(container, SwingUtilities.computeUnion(x, y, width, height, rectangle));
                }
            }
            this.scheduleProcessingRunnable(appContext);
        }
    }
    
    void nativeQueueSurfaceDataRunnable(AppContext appContext, final Component component, final Runnable runnable) {
        synchronized (this) {
            if (this.runnableList == null) {
                this.runnableList = new LinkedList();
            }
            this.runnableList.add(new Runnable() {
                @Override
                public void run() {
                    RepaintManager.javaSecurityAccess.doIntersectionPrivilege(new PrivilegedAction<Void>() {
                        @Override
                        public Void run() {
                            runnable.run();
                            return null;
                        }
                    }, AccessController.getContext(), AWTAccessor.getComponentAccessor().getAccessControlContext(component));
                }
            });
        }
        this.scheduleProcessingRunnable(appContext);
    }
    
    private synchronized boolean extendDirtyRegion(Component component, int n, int n2, int n3, int n4) {
        Rectangle rectangle = (Rectangle)this.dirtyComponents.get(component);
        if (rectangle != null) {
            SwingUtilities.computeUnion(n, n2, n3, n4, rectangle);
            return true;
        }
        return false;
    }
    
    public Rectangle getDirtyRegion(JComponent component) {
        RepaintManager delegate = this.getDelegate(component);
        if (delegate != null) {
            return delegate.getDirtyRegion(component);
        }
        Rectangle r;
        synchronized (this) {
            r = (Rectangle)this.dirtyComponents.get(component);
        }
        if (r == null) {
            return new Rectangle(0, 0, 0, 0);
        }
        return new Rectangle(r);
    }
    
    public void markCompletelyDirty(JComponent component) {
        RepaintManager delegate = this.getDelegate(component);
        if (delegate != null) {
            delegate.markCompletelyDirty(component);
            return;
        }
        this.addDirtyRegion(component, 0, 0, Integer.MAX_VALUE, Integer.MAX_VALUE);
        return;
    }
    
    public void markCompletelyClean(JComponent component) {
        RepaintManager delegate = this.getDelegate(component);
        if (delegate != null) {
            delegate.markCompletelyClean(component);
            return;
        }
        synchronized (this) {
            this.dirtyComponents.remove(component);
        }
        return;
    }
    
    public boolean isCompletelyDirty(JComponent component) {
        RepaintManager delegate = this.getDelegate(component);
        if (delegate != null) {
            return delegate.isCompletelyDirty(component);
        }
        Rectangle dirtyRegion = this.getDirtyRegion(component);
        return dirtyRegion.width == Integer.MAX_VALUE && dirtyRegion.height == Integer.MAX_VALUE;
    }
    
    public void validateInvalidComponents() {
        List<Object> invalidComponents;
        synchronized (this) {
            if (this.invalidComponents == null) {
                return;
            }
            invalidComponents = this.invalidComponents;
            this.invalidComponents = null;
        }
        for (int size = invalidComponents.size(), i = 0; i < size; ++i) {
            final Component component = (Component)invalidComponents.get(i);
            RepaintManager.javaSecurityAccess.doIntersectionPrivilege(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    component.validate();
                    return null;
                }
            }, AccessController.getContext(), AWTAccessor.getComponentAccessor().getAccessControlContext(component));
        }
    }
    
    private void prePaintDirtyRegions() {
        Map<Component, Rectangle> dirtyComponents;
        List<Object> runnableList;
        synchronized (this) {
            dirtyComponents = this.dirtyComponents;
            runnableList = this.runnableList;
            this.runnableList = null;
        }
        if (runnableList != null) {
            Iterator<Object> iterator = runnableList.iterator();
            while (iterator.hasNext()) {
                ((Runnable)iterator.next()).run();
            }
        }
        this.paintDirtyRegions();
        if (dirtyComponents.size() > 0) {
            this.paintDirtyRegions(dirtyComponents);
        }
    }
    
    private void updateWindows(Map<Component, Rectangle> map) {
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        if (!(defaultToolkit instanceof SunToolkit) || !((SunToolkit)defaultToolkit).needUpdateWindow()) {
            return;
        }
        HashSet<Window> set = new HashSet();
        for (Component component : map.keySet()) {
            Window window = (component instanceof Window) ? ((Window)component) : SwingUtilities.getWindowAncestor(component);
            if (window != null && !window.isOpaque()) {
                set.add(window);
            }
            else {
                continue;
            }
        }
        Iterator<Object> iterator2 = set.iterator();
        while (iterator2.hasNext()) {
            AWTAccessor.getWindowAccessor().updateWindow((Window)iterator2.next());
        }
        return;
    }
    
    boolean isPainting() {
        return this.painting;
    }
    
    public void paintDirtyRegions() {
        synchronized (this) {
            Map<Object, Object> tmpDirtyComponents = this.tmpDirtyComponents;
            this.tmpDirtyComponents = this.dirtyComponents;
            (this.dirtyComponents = tmpDirtyComponents).clear();
        }
        this.paintDirtyRegions(this.tmpDirtyComponents);
    }
    
    private void paintDirtyRegions(final Map<Component, Rectangle> map) {
        if (map.isEmpty()) {
            return;
        }
        final ArrayList<Component> list = new ArrayList(map.size());
        Iterator<Object> iterator = map.keySet().iterator();
        while (iterator.hasNext()) {
            this.collectDirtyComponents(map, (Component)iterator.next(), list);
        }
        final Object o = new AtomicInteger(list.size());
        this.painting = true;
        try {
            for (int i = 0; i < o.get(); ++i) {
                final Component component = (Component)list.get(i);
                RepaintManager.javaSecurityAccess.doIntersectionPrivilege(new PrivilegedAction<Void>() {
                    @Override
                    public Void run() {
                        Rectangle rectangle = (Rectangle)map.get(component);
                        if (rectangle == null) {
                            return null;
                        }
                        SwingUtilities.computeIntersection(0, 0, component.getWidth(), component.getHeight(), rectangle);
                        if ((component instanceof JComponent)) {
                            ((JComponent)component).paintImmediately(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
                        }
                        else if (component.isShowing()) {
                            Graphics safelyGetGraphics = JComponent.safelyGetGraphics(component, component);
                            if (safelyGetGraphics != null) {
                                safelyGetGraphics.setClip(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
                                try {
                                    component.paint(safelyGetGraphics);
                                }
                                finally {
                                    safelyGetGraphics.dispose();
                                }
                            }
                        }
                        if (RepaintManager.this.repaintRoot != null) {
                            RepaintManager.access$500(RepaintManager.this, RepaintManager.this.repaintRoot, list, i + 1);
                            o.set(list.size());
                            RepaintManager.this.paintManager.isRepaintingRoot = true;
                            RepaintManager.this.repaintRoot.paintImmediately(0, 0, RepaintManager.this.repaintRoot.getWidth(), RepaintManager.this.repaintRoot.getHeight());
                            RepaintManager.this.paintManager.isRepaintingRoot = false;
                            RepaintManager.this.repaintRoot = null;
                        }
                        return null;
                    }
                }, AccessController.getContext(), AWTAccessor.getComponentAccessor().getAccessControlContext(component));
            }
        }
        finally {
            this.painting = false;
        }
        this.updateWindows(map);
        map.clear();
        return;
    }
    
    private void adjustRoots(JComponent component, List<Component> list, int n) {
        for (int i = list.size() - 1; i >= n; --i) {
            Component parent;
            for (parent = (Component)list.get(i); parent != component && parent != null && parent instanceof JComponent; parent = parent.getParent()) {}
            if (parent == component) {
                list.remove(i);
            }
        }
    }
    
    void collectDirtyComponents(Map<Component, Rectangle> map, Component component, List<Component> list) {
        Component component2 = component;
        Component component3 = component;
        int n = component.getX();
        int n2 = component.getY();
        int width = component.getWidth();
        int height = component.getHeight();
        int n4;
        int n3 = n4 = 0;
        int n6;
        int n5 = n6 = 0;
        this.tmp.setBounds((Rectangle)map.get(component));
        SwingUtilities.computeIntersection(0, 0, width, height, this.tmp);
        if (this.tmp.isEmpty()) {
            return;
        }
        Label_0237: {
            while (component3 instanceof JComponent) {
                Container parent = component3.getParent();
                if (parent == null) {
                    if (component != component2) {
                        this.tmp.setLocation(this.tmp.x + n3 - n4, this.tmp.y + n5 - n6);
                        SwingUtilities.computeUnion(this.tmp.x, this.tmp.y, this.tmp.width, this.tmp.height, (Rectangle)map.get(component2));
                    }
                    if (!list.contains(component2)) {
                        list.add(component2);
                    }
                    return;
                }
                else {
                    component3 = parent;
                    n4 += n;
                    n6 += n2;
                    this.tmp.setLocation(this.tmp.x + n, this.tmp.y + n2);
                    n = component3.getX();
                    n2 = component3.getY();
                    this.tmp = SwingUtilities.computeIntersection(0, 0, component3.getWidth(), component3.getHeight(), this.tmp);
                    if (this.tmp.isEmpty()) {
                        return;
                    }
                    if (map.get(component3) == null) {
                        continue;
                    }
                    component2 = component3;
                    n3 = n4;
                    n5 = n6;
                }
            }
            continue;
        }
    }
    
    @Override
    public synchronized String toString() {
        StringBuffer sb = new StringBuffer();
        if (this.dirtyComponents != null) {
            sb.append("" + this.dirtyComponents);
        }
        return sb.toString();
    }
    
    public Image getOffscreenBuffer(Component component, int n, int n2) {
        RepaintManager delegate = this.getDelegate(component);
        if (delegate != null) {
            return delegate.getOffscreenBuffer(component, n, n2);
        }
        return this._getOffscreenBuffer(component, n, n2);
    }
    
    public Image getVolatileOffscreenBuffer(Component component, int width, int height) {
        RepaintManager delegate = this.getDelegate(component);
        if (delegate != null) {
            return delegate.getVolatileOffscreenBuffer(component, width, height);
        }
        if (!((component instanceof Window) ? ((Window)component) : SwingUtilities.getWindowAncestor(component)).isOpaque()) {
            Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
            if (defaultToolkit instanceof SunToolkit && ((SunToolkit)defaultToolkit).needUpdateWindow()) {
                return null;
            }
        }
        GraphicsConfiguration graphicsConfiguration = component.getGraphicsConfiguration();
        if (graphicsConfiguration == null) {
            graphicsConfiguration = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
        }
        Dimension doubleBufferMaximumSize = this.getDoubleBufferMaximumSize();
        width = ((width < 1) ? 1 : ((width > doubleBufferMaximumSize.width) ? doubleBufferMaximumSize.width : width));
        height = ((height < 1) ? 1 : ((height > doubleBufferMaximumSize.height) ? doubleBufferMaximumSize.height : height));
        VolatileImage compatibleVolatileImage = (VolatileImage)this.volatileMap.get(graphicsConfiguration);
        if (compatibleVolatileImage == null || compatibleVolatileImage.getWidth() < width || compatibleVolatileImage.getHeight() < height) {
            if (compatibleVolatileImage != null) {
                compatibleVolatileImage.flush();
            }
            compatibleVolatileImage = graphicsConfiguration.createCompatibleVolatileImage(width, height, RepaintManager.volatileBufferType);
            this.volatileMap.put(graphicsConfiguration, compatibleVolatileImage);
            return compatibleVolatileImage;
        }
        else {
            return compatibleVolatileImage;
        }
    }
    
    private Image _getOffscreenBuffer(Component component, int max, int max2) {
        Dimension doubleBufferMaximumSize = this.getDoubleBufferMaximumSize();
        if (!((component instanceof Window) ? ((Window)component) : SwingUtilities.getWindowAncestor(component)).isOpaque()) {
            Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
            if (defaultToolkit instanceof SunToolkit && ((SunToolkit)defaultToolkit).needUpdateWindow()) {
                return null;
            }
        }
        if (this.standardDoubleBuffer == null) {
            this.standardDoubleBuffer = new RepaintManager.DoubleBufferInfo();
        }
        RepaintManager.DoubleBufferInfo standardDoubleBuffer = this.standardDoubleBuffer;
        max = ((max < 1) ? 1 : ((max > doubleBufferMaximumSize.width) ? doubleBufferMaximumSize.width : max));
        max2 = ((max2 < 1) ? 1 : ((max2 > doubleBufferMaximumSize.height) ? doubleBufferMaximumSize.height : max2));
        if (standardDoubleBuffer.needsReset || (standardDoubleBuffer.image != null && (standardDoubleBuffer.size.width < max || standardDoubleBuffer.size.height < max2))) {
            standardDoubleBuffer.needsReset = false;
            if (standardDoubleBuffer.image != null) {
                standardDoubleBuffer.image.flush();
                standardDoubleBuffer.image = null;
            }
            max = Math.max(standardDoubleBuffer.size.width, max);
            max2 = Math.max(standardDoubleBuffer.size.height, max2);
        }
        Image image = standardDoubleBuffer.image;
        if (standardDoubleBuffer.image == null) {
            image = component.createImage(max, max2);
            standardDoubleBuffer.size = new Dimension(max, max2);
            if ((component instanceof JComponent)) {
                ((JComponent)component).setCreatedDoubleBuffer(true);
                standardDoubleBuffer.image = image;
            }
        }
        return image;
    }
    
    public void setDoubleBufferMaximumSize(Dimension doubleBufferMaxSize) {
        this.doubleBufferMaxSize = doubleBufferMaxSize;
        if (this.doubleBufferMaxSize == null) {
            this.clearImages();
        }
        else {
            this.clearImages(doubleBufferMaxSize.width, doubleBufferMaxSize.height);
        }
    }
    
    private void clearImages() {
        this.clearImages(0, 0);
    }
    
    private void clearImages(int n, int n2) {
        if (this.standardDoubleBuffer != null && this.standardDoubleBuffer.image != null && (this.standardDoubleBuffer.image.getWidth(null) > n || this.standardDoubleBuffer.image.getHeight(null) > n2)) {
            this.standardDoubleBuffer.image.flush();
            this.standardDoubleBuffer.image = null;
        }
        Iterator<GraphicsConfiguration> iterator = this.volatileMap.keySet().iterator();
        while (iterator.hasNext()) {
            VolatileImage volatileImage = (VolatileImage)this.volatileMap.get((Object)iterator.next());
            if (volatileImage.getWidth() > n || volatileImage.getHeight() > n2) {
                volatileImage.flush();
                iterator.remove();
            }
            else {
                continue;
            }
        }
    }
    
    public Dimension getDoubleBufferMaximumSize() {
        if (this.doubleBufferMaxSize == null) {
            try {
                Rectangle union = new Rectangle();
                GraphicsDevice[] screenDevices = GraphicsEnvironment.getLocalGraphicsEnvironment().getScreenDevices();
                for (int length = screenDevices.length, i = 0; i < length; ++i) {
                    union = union.union(screenDevices[i].getDefaultConfiguration().getBounds());
                }
                this.doubleBufferMaxSize = new Dimension(union.width, union.height);
            }
            catch (HeadlessException ex) {
                this.doubleBufferMaxSize = new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
            }
        }
        return this.doubleBufferMaxSize;
    }
    
    public void setDoubleBufferingEnabled(boolean doubleBufferingEnabled) {
        this.doubleBufferingEnabled = doubleBufferingEnabled;
        RepaintManager.PaintManager paintManager = this.getPaintManager();
        if (!doubleBufferingEnabled && paintManager.getClass() != RepaintManager.PaintManager.class) {
            this.setPaintManager(new RepaintManager.PaintManager());
        }
    }
    
    public boolean isDoubleBufferingEnabled() {
        return this.doubleBufferingEnabled;
    }
    
    void resetDoubleBuffer() {
        if (this.standardDoubleBuffer != null) {
            this.standardDoubleBuffer.needsReset = true;
        }
    }
    
    void resetVolatileDoubleBuffer(GraphicsConfiguration graphicsConfiguration) {
        Image image = (Image)this.volatileMap.remove(graphicsConfiguration);
        if (image != null) {
            image.flush();
        }
    }
    
    boolean useVolatileDoubleBuffer() {
        return RepaintManager.volatileImageBufferEnabled;
    }
    
    private synchronized boolean isPaintingThread() {
        return Thread.currentThread() == this.paintThread;
    }
    
    void paint(JComponent component, JComponent component2, Graphics graphics, int n, int n2, int n3, int n4) {
        RepaintManager.PaintManager paintManager = this.getPaintManager();
        if (!this.isPaintingThread() && paintManager.getClass() != RepaintManager.PaintManager.class) {
            paintManager = new RepaintManager.PaintManager();
            paintManager.repaintManager = this;
        }
        if (!paintManager.paint(component, component2, graphics, n, n2, n3, n4)) {
            graphics.setClip(n, n2, n3, n4);
            component.paintToOffscreen(graphics, n, n2, n3, n4, n + n3, n2 + n4);
        }
    }
    
    void copyArea(JComponent component, Graphics graphics, int n, int n2, int n3, int n4, int n5, int n6, boolean b) {
        this.getPaintManager().copyArea(component, graphics, n, n2, n3, n4, n5, n6, b);
    }
    
    private void addRepaintListener(SwingUtilities2.RepaintListener repaintListener) {
        this.repaintListeners.add(repaintListener);
    }
    
    private void removeRepaintListener(SwingUtilities2.RepaintListener repaintListener) {
        this.repaintListeners.remove(repaintListener);
    }
    
    void notifyRepaintPerformed(JComponent component, int n, int n2, int n3, int n4) {
        Iterator<SwingUtilities2.RepaintListener> iterator = this.repaintListeners.iterator();
        while (iterator.hasNext()) {
            ((SwingUtilities2.RepaintListener)iterator.next()).repaintPerformed(component, n, n2, n3, n4);
        }
    }
    
    void beginPaint() {
        boolean n = false;
        Thread currentThread = Thread.currentThread();
        int paintDepth;
        synchronized (this) {
            paintDepth = this.paintDepth;
            if (this.paintThread == null || currentThread == this.paintThread) {
                this.paintThread = currentThread;
                ++this.paintDepth;
            }
            else {
                n = true;
            }
        }
        if (n == 0 && paintDepth == 0) {
            this.getPaintManager().beginPaint();
        }
    }
    
    void endPaint() {
        if (this.isPaintingThread()) {
            RepaintManager.PaintManager paintManager = null;
            synchronized (this) {
                int paintDepth = this.paintDepth - 1;
                this.paintDepth = paintDepth;
                if (paintDepth == 0) {
                    paintManager = this.getPaintManager();
                }
            }
            if (paintManager != null) {
                paintManager.endPaint();
                synchronized (this) {
                    this.paintThread = null;
                }
            }
        }
    }
    
    boolean show(Container container, int n, int n2, int n3, int n4) {
        return this.getPaintManager().show(container, n, n2, n3, n4);
    }
    
    void doubleBufferingChanged(JRootPane rootPane) {
        this.getPaintManager().doubleBufferingChanged(rootPane);
    }
    
    void setPaintManager(RepaintManager.PaintManager paintManager) {
        if (paintManager == null) {
            paintManager = new RepaintManager.PaintManager();
        }
        RepaintManager.PaintManager paintManager2;
        synchronized (this) {
            paintManager2 = this.paintManager;
            this.paintManager = paintManager;
            paintManager.repaintManager = this;
        }
        if (paintManager2 != null) {
            paintManager2.dispose();
        }
    }
    
    private synchronized RepaintManager.PaintManager getPaintManager() {
        if (this.paintManager == null) {
            BufferStrategyPaintManager paintManager = null;
            if (this.doubleBufferingEnabled && !RepaintManager.nativeDoubleBuffering) {
                switch (this.bufferStrategyType) {
                    case 0: {
                        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
                        if ((defaultToolkit instanceof SunToolkit)) {
                            if (((SunToolkit)defaultToolkit).useBufferPerWindow()) {
                                paintManager = new BufferStrategyPaintManager();
                            }
                            break;
                        }
                        else {
                            break;
                        }
                    }
                    case 1: {
                        paintManager = new BufferStrategyPaintManager();
                        break;
                    }
                }
            }
            this.setPaintManager(paintManager);
        }
        return this.paintManager;
    }
    
    private void scheduleProcessingRunnable(AppContext appContext) {
        if (this.processingRunnable.markPending()) {
            if ((Toolkit.getDefaultToolkit() instanceof SunToolkit)) {
                SunToolkit.getSystemEventQueueImplPP(appContext).postEvent(new InvocationEvent(Toolkit.getDefaultToolkit(), this.processingRunnable));
            }
            else {
                Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(new InvocationEvent(Toolkit.getDefaultToolkit(), this.processingRunnable));
            }
        }
    }
    
    private RepaintManager getDelegate(Component component) {
        RepaintManager delegateRepaintManager = SwingUtilities3.getDelegateRepaintManager(component);
        if (this == delegateRepaintManager) {
            delegateRepaintManager = null;
            return delegateRepaintManager;
        }
        return delegateRepaintManager;
    }
    
    static void access$000(RepaintManager repaintManager, SwingUtilities2.RepaintListener repaintListener) {
        repaintManager.addRepaintListener(repaintListener);
    }
    
    static void access$100(RepaintManager repaintManager, SwingUtilities2.RepaintListener repaintListener) {
        repaintManager.removeRepaintListener(repaintListener);
    }
    
    static void access$500(RepaintManager repaintManager, JComponent component, List list, int n) {
        repaintManager.adjustRoots(component, list, n);
    }
    
    static void access$1100(RepaintManager repaintManager) {
        repaintManager.displayChanged();
    }
    
    static void access$1200(RepaintManager repaintManager) {
        repaintManager.prePaintDirtyRegions();
    }
    
    static {
        repaintManagerKey = RepaintManager.class;
        RepaintManager.volatileImageBufferEnabled = true;
        javaSecurityAccess = SharedSecrets.getJavaSecurityAccess();
        displayChangedHandler = new RepaintManager.DisplayChangedHandler();
        SwingAccessor.setRepaintManagerAccessor(new SwingAccessor.RepaintManagerAccessor() {
            @Override
            public void addRepaintListener(RepaintManager repaintManager, SwingUtilities2.RepaintListener repaintListener) {
                RepaintManager.access$000(repaintManager, repaintListener);
            }
            
            @Override
            public void removeRepaintListener(RepaintManager repaintManager, SwingUtilities2.RepaintListener repaintListener) {
                RepaintManager.access$100(repaintManager, repaintListener);
            }
        });
        RepaintManager.volatileImageBufferEnabled = "true".equals(AccessController.doPrivileged(new GetPropertyAction("swing.volatileImageBufferEnabled", "true")));
        int headless = GraphicsEnvironment.isHeadless();
        if (RepaintManager.volatileImageBufferEnabled && headless) {
            RepaintManager.volatileImageBufferEnabled = false;
        }
        RepaintManager.nativeDoubleBuffering = "true".equals(AccessController.doPrivileged(new GetPropertyAction("awt.nativeDoubleBuffering")));
        String anObject = (String)AccessController.doPrivileged(new GetPropertyAction("swing.bufferPerWindow"));
        if (!(headless == 0)) {
            BUFFER_STRATEGY_TYPE = 2;
        }
        else if (anObject == null) {
            BUFFER_STRATEGY_TYPE = 0;
        }
        else if ("true".equals(anObject)) {
            BUFFER_STRATEGY_TYPE = 1;
        }
        else {
            BUFFER_STRATEGY_TYPE = 2;
        }
        HANDLE_TOP_LEVEL_PAINT = "true".equals(AccessController.doPrivileged(new GetPropertyAction("swing.handleTopLevelPaint", "true")));
        GraphicsEnvironment localGraphicsEnvironment = GraphicsEnvironment.getLocalGraphicsEnvironment();
        if ((localGraphicsEnvironment instanceof SunGraphicsEnvironment)) {
            ((SunGraphicsEnvironment)localGraphicsEnvironment).addDisplayChangedListener(RepaintManager.displayChangedHandler);
        }
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        if (defaultToolkit instanceof SunToolkit && ((SunToolkit)defaultToolkit).isSwingBackbufferTranslucencySupported()) {
            volatileBufferType = 3;
        }
        else {
            volatileBufferType = 1;
        }
    }
    
    static class PaintManager
    {
        protected RepaintManager repaintManager;
        boolean isRepaintingRoot;
        
        public boolean paint(JComponent component, JComponent component2, Graphics graphics, int n, int n2, int n3, int n4) {
            boolean n5 = false;
            Image validImage;
            if (this.repaintManager.useVolatileDoubleBuffer() && (validImage = this.getValidImage(this.repaintManager.getVolatileOffscreenBuffer(component2, n3, n4))) != null) {
                VolatileImage volatileImage = (VolatileImage)validImage;
                GraphicsConfiguration graphicsConfiguration = component2.getGraphicsConfiguration();
                for (int n6 = 0; n5 == 0 && n6 < 2; n5 = !volatileImage.contentsLost(), ++n6) {
                    if (volatileImage.validate(graphicsConfiguration) == 2) {
                        this.repaintManager.resetVolatileDoubleBuffer(graphicsConfiguration);
                        volatileImage = (VolatileImage)this.repaintManager.getVolatileOffscreenBuffer(component2, n3, n4);
                    }
                    this.paintDoubleBuffered(component, volatileImage, graphics, n, n2, n3, n4);
                }
            }
            Image validImage2;
            if (n5 == 0 && (validImage2 = this.getValidImage(this.repaintManager.getOffscreenBuffer(component2, n3, n4))) != null) {
                this.paintDoubleBuffered(component, validImage2, graphics, n, n2, n3, n4);
                n5 = true;
            }
            return n5;
        }
        
        public void copyArea(JComponent component, Graphics graphics, int n, int n2, int n3, int n4, int n5, int n6, boolean b) {
            graphics.copyArea(n, n2, n3, n4, n5, n6);
        }
        
        public void beginPaint() {
        }
        
        public void endPaint() {
        }
        
        public boolean show(Container container, int n, int n2, int n3, int n4) {
            return false;
        }
        
        public void doubleBufferingChanged(JRootPane rootPane) {
        }
        
        protected void paintDoubleBuffered(JComponent component, Image image, Graphics graphics, int i, int j, int a, int a2) {
            Graphics graphics2 = image.getGraphics();
            int min = Math.min(a, image.getWidth(null));
            int min2 = Math.min(a2, image.getHeight(null));
            try {
                for (int n = i + a; i < n; i += min) {
                    for (int n2 = j + a2; j < n2; j += min2) {
                        graphics2.translate(-i, -j);
                        graphics2.setClip(i, j, min, min2);
                        if (RepaintManager.volatileBufferType != 1 && graphics2 instanceof Graphics2D) {
                            Graphics2D graphics2D = (Graphics2D)graphics2;
                            Color background = graphics2D.getBackground();
                            graphics2D.setBackground(component.getBackground());
                            graphics2D.clearRect(i, j, min, min2);
                            graphics2D.setBackground(background);
                        }
                        component.paintToOffscreen(graphics2, i, j, min, min2, n, n2);
                        graphics.setClip(i, j, min, min2);
                        if (RepaintManager.volatileBufferType != 1 && graphics instanceof Graphics2D) {
                            Graphics2D graphics2D2 = (Graphics2D)graphics;
                            Composite composite = graphics2D2.getComposite();
                            graphics2D2.setComposite(AlphaComposite.Src);
                            graphics2D2.drawImage(image, i, j, component);
                            graphics2D2.setComposite(composite);
                        }
                        else {
                            graphics.drawImage(image, i, j, component);
                        }
                        graphics2.translate(i, j);
                    }
                }
            }
            finally {
                graphics2.dispose();
            }
        }
        
        private Image getValidImage(Image image) {
            if (image != null && image.getWidth(null) > 0 && image.getHeight(null) > 0) {
                return image;
            }
            return null;
        }
        
        protected void repaintRoot(JComponent component) {
            assert this.repaintManager.repaintRoot == null;
            if (this.repaintManager.painting) {
                this.repaintManager.repaintRoot = component;
            }
            else {
                component.repaint();
            }
            return;
        }
        
        protected boolean isRepaintingRoot() {
            return this.isRepaintingRoot;
        }
        
        protected void dispose() {
        }
    }
    
    private class DoubleBufferInfo
    {
        public Image image;
        public Dimension size;
        public boolean needsReset;
        
        private DoubleBufferInfo() {
            this.needsReset = false;
        }
    }
    
    private static final class DisplayChangedHandler implements DisplayChangedListener
    {
        DisplayChangedHandler() {
        }
        
        @Override
        public void displayChanged() {
            scheduleDisplayChanges();
        }
        
        @Override
        public void paletteChanged() {
        }
        
        private static void scheduleDisplayChanges() {
            for (AppContext appContext : AppContext.getAppContexts()) {
                synchronized (appContext) {
                    if (!appContext.isDisposed()) {
                        EventQueue eventQueue = (EventQueue)appContext.get(AppContext.EVENT_QUEUE_KEY);
                        if (eventQueue != null) {
                            eventQueue.postEvent(new InvocationEvent(Toolkit.getDefaultToolkit(), new RepaintManager.DisplayChangedRunnable()));
                        }
                        else {
                            continue;
                        }
                    }
                    else {
                        continue;
                    }
                }
            }
        }
    }
    
    private static final class DisplayChangedRunnable implements Runnable
    {
        @Override
        public void run() {
            RepaintManager.access$1100(RepaintManager.currentManager((JComponent)null));
        }
    }
    
    private final class ProcessingRunnable implements Runnable
    {
        private boolean pending;
        
        public synchronized boolean markPending() {
            return !this.pending && (this.pending = true);
        }
        
        @Override
        public void run() {
            synchronized (this) {
                this.pending = false;
            }
            RepaintManager.this.scheduleHeavyWeightPaints();
            RepaintManager.this.validateInvalidComponents();
            RepaintManager.access$1200(RepaintManager.this);
        }
    }
}
